package main

import (
	"flag"
	"os"

	"github.com/rs/zerolog"
	"github.com/rs/zerolog/log"

	"code.chrissexton.org/cws/cabinet/auth"
	"code.chrissexton.org/cws/cabinet/db"
	"code.chrissexton.org/cws/cabinet/entry"
	"code.chrissexton.org/cws/cabinet/web"

	_ "github.com/mattn/go-sqlite3"
)

var (
	dbPath      = flag.String("db", "cabinet.db", "path to db")
	httpAddr    = flag.String("httpAddr", "0.0.0.0:8080", "http address")
	salt        = flag.String("salt", "c4b1n3t", "salt for IDs")
	minHashLen  = flag.Int("minHashLen", 4, "minimum ID hash size")
	develop     = flag.Bool("develop", false, "turn on develop mode")
	adminSecret = flag.String("adminSecret", "helpme123", "secret for user creation")
)

func main() {
	flag.Parse()
	log.Logger = log.Logger.Output(zerolog.ConsoleWriter{Out: os.Stderr})
	zerolog.SetGlobalLevel(zerolog.DebugLevel)

	db, err := db.New(*dbPath)
	if err != nil {
		log.Fatal().
			Err(err).
			Msg("could not connect to database")
	}

	tx := db.MustBegin()
	if err := entry.PrepareTable(tx); err != nil {
		log.Fatal().Err(err).Msg("could not create database")
	}
	if err = auth.PrepareTable(tx); err != nil {
		log.Fatal().Err(err).Msg("could not create database")
	}
	tx.Commit()

	s := web.New(*httpAddr, db, FS(*develop))
	s.Serve()
}
